package com.bytesw.bytebot.model;

import com.bytesw.bytebot.model.converters.AgentStatusConverter;
import com.bytesw.bytebot.model.converters.AgentTypeConverter;
import com.bytesw.bytebot.model.converters.LanguageConverter;
import com.bytesw.bytebot.model.converters.StatusConverter;
import com.bytesw.bytebot.model.enums.AgentStatusEnum;
import com.bytesw.bytebot.model.enums.AgentTypeEnum;
import com.bytesw.bytebot.model.enums.LanguageEnum;
import com.bytesw.bytebot.model.enums.StatusEnum;
import lombok.*;
import org.hibernate.annotations.Type;

import javax.persistence.*;
import java.io.Serializable;
import java.util.List;

@Entity
@Getter @Setter @ToString @EqualsAndHashCode(of = "id")
@NoArgsConstructor
@Table(name = "AVB_AGENT")
@NamedQuery(name = "Agent.findByPK", query = "Select u from Agent u where u.id = ?1")
public class Agent implements Serializable {

    @Id
    @Column(name = "AGEN_ID")
    @GeneratedValue(strategy = GenerationType.TABLE, generator = "AVB_AGENT_GENERATOR")
    @TableGenerator(name = "AVB_AGENT_GENERATOR", table = "SEQUENCE_TABLE", pkColumnName = "SEQ_NAME", valueColumnName = "SEQ_COUNT",
            pkColumnValue = "AVB_AGENT_SEQ", allocationSize = 1)
    private Long id;

    @Column(name = "AGEN_IDEN")
    private String name;

    @Column(name = "AGEN_DESCR")
    private String description;

    @Column(name = "AGEN_VERS")
    private String version;

    @Column(name = "AGEN_LANG")
    @Convert(converter =  LanguageConverter.class)
    private LanguageEnum language;

    @Column(name = "AGEN_TZONE")
    private String timezone;

    @Lob
    @Type(type = "org.hibernate.type.TextType")
    @Basic(fetch = FetchType.LAZY)
    @Column(name = "AGEN_AVAT")
    private String avatar;

    @ManyToOne
    @JoinColumn(name = "COUN_ID", referencedColumnName = "COUN_ID")
    private Country country;

    @Column(name = "AGEN_STATE")
    @Convert(converter =  AgentStatusConverter.class)
    private AgentStatusEnum status;

    @Column(name = "AGEN_TYPE")
    @Convert(converter =  AgentTypeConverter.class)
    private AgentTypeEnum type;

    @OneToMany(mappedBy = "agent")
    private List<DeploymentChannel> deploymentChannels;

    @OneToMany(mappedBy = "agent")
    private List<FrequentQuestion> frequentQuestions;
}
