package com.bytesw.bytebot.service.dashboard;

import com.bytesw.bytebot.bean.AverageBean;
import com.bytesw.bytebot.bean.RangeBean;
import com.bytesw.bytebot.bean.SummaryBean;
import com.bytesw.bytebot.jdbc.BotSessionJDCBRepository;
import com.bytesw.bytebot.jdbc.MessageJDBCRepository;
import com.bytesw.bytebot.jdbc.ResponseJDBCRepository;
import com.bytesw.xdf.exception.NotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.time.Duration;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class OperativeDashboardService extends DashboardService {

    @Autowired
    private BotSessionJDCBRepository botSessionJDCBRepository;

    @Autowired
    private MessageJDBCRepository messageJDBCRepository;

    @Autowired
    private ResponseJDBCRepository responseJDBCRepository;

    @Override
    protected void completeData(OffsetDateTime startDate, OffsetDateTime endDate, Map<String, Object> info) {
        List<RangeBean> rangeList = calculateRangeList(startDate, endDate);
        List<Object[]> messageByActivity = generateData(rangeList);

        //Cantidad de sesiones
        int cantSessions = botSessionJDCBRepository.countSessionInRange(startDate, endDate);

        int cantReceivedMessages = messageJDBCRepository.countSessionInRange(startDate, endDate);

        int cantResponseMessages = responseJDBCRepository.countSessionInRange(startDate, endDate);

        //Promedio primera respuesta en mili
        Object avgFRObject = botSessionJDCBRepository.getAvgFirstResponseTime(startDate, endDate);
        Double avgFirstResponse;
        if (avgFRObject instanceof Long) {
            avgFirstResponse = new Double((Long)avgFRObject);
        } else {
            avgFirstResponse = (Double) avgFRObject;
        }
        Double avgSessionsByCustomer = botSessionJDCBRepository.getAvgSessionByCustomerInRange(startDate, endDate);

        Timestamp timestamp = botSessionJDCBRepository.getLastDateInRage(startDate, endDate);

        if (timestamp == null) {
            throw new NotFoundException("No se encontró una sesión en el rango de fechas indicado");
        }

        //Calculando el tiempo de inactividad de la sesion
        OffsetDateTime lastSessionDate = OffsetDateTime.ofInstant(timestamp.toInstant(), ZoneId.systemDefault());
        Duration difference = Duration.between(lastSessionDate, endDate);

        SummaryBean sessionInactivity = new SummaryBean();
        sessionInactivity.setValue(new BigInteger(String.valueOf(difference.getSeconds())));
        sessionInactivity.setHistory(new ArrayList<>());
        sessionInactivity.setPercent(new BigDecimal("1.2"));
        sessionInactivity.getHistory().add(new BigInteger("10"));
        sessionInactivity.getHistory().add(new BigInteger("10"));
        sessionInactivity.getHistory().add(new BigInteger("10"));
        sessionInactivity.getHistory().add(new BigInteger("10"));
        sessionInactivity.getHistory().add(new BigInteger("10"));


        SummaryBean totalSessions = new SummaryBean();
        totalSessions.setValue(new BigInteger(String.valueOf(cantSessions)));
        totalSessions.setHistory(new ArrayList<>());
        totalSessions.setPercent(new BigDecimal("1.2"));
        totalSessions.getHistory().add(new BigInteger("2"));
        totalSessions.getHistory().add(new BigInteger("4"));
        totalSessions.getHistory().add(new BigInteger("6"));
        totalSessions.getHistory().add(new BigInteger("8"));
        totalSessions.getHistory().add(new BigInteger("10"));

        SummaryBean totalReceivedMessages = new SummaryBean();
        totalReceivedMessages.setValue(new BigInteger(String.valueOf(cantReceivedMessages)));
        totalReceivedMessages.setHistory(new ArrayList<>());
        totalReceivedMessages.setPercent(new BigDecimal("1.2"));
        totalReceivedMessages.getHistory().add(new BigInteger("10"));
        totalReceivedMessages.getHistory().add(new BigInteger("8"));
        totalReceivedMessages.getHistory().add(new BigInteger("6"));
        totalReceivedMessages.getHistory().add(new BigInteger("8"));
        totalReceivedMessages.getHistory().add(new BigInteger("10"));

        SummaryBean totalSentMessages = new SummaryBean();
        totalSentMessages.setValue(new BigInteger(String.valueOf(cantResponseMessages)));
        totalSentMessages.setHistory(new ArrayList<>());
        totalSentMessages.setPercent(new BigDecimal("1.2"));
        totalSentMessages.getHistory().add(new BigInteger("2"));
        totalSentMessages.getHistory().add(new BigInteger("5"));
        totalSentMessages.getHistory().add(new BigInteger("10"));
        totalSentMessages.getHistory().add(new BigInteger("5"));
        totalSentMessages.getHistory().add(new BigInteger("2"));


        Map<String, Object> summary = new HashMap<>();
        summary.put("sessionInactivity", sessionInactivity);
        summary.put("totalSessions", totalSessions);
        summary.put("totalReceivedMessages", totalReceivedMessages);
        summary.put("totalSentMessages", totalSentMessages);


        AverageBean averageBean = new AverageBean();
        if (avgFirstResponse == null) {
            averageBean.setFirstResponseAverage(new BigDecimal(0));
        } else {
            averageBean.setFirstResponseAverage(new BigDecimal(avgFirstResponse));
        }
        averageBean.setSessionAverage(new BigDecimal(avgSessionsByCustomer));
        info.put("summary", summary);
        info.put("averages", averageBean);
        info.put("customerMessageDetail", messageByActivity);
    }

    private List<Object[]> generateData(List<RangeBean> rangeList) {
        List<Object[]> data = new ArrayList<>();
        Map<Long, Map<Integer, Integer>> accumulated = new HashMap<>();
        for (RangeBean range : rangeList) {

            List<RangeBean> hourRanges = getRangeHours(range.getStartDate(), range.getEndDate());
            for (RangeBean hourRange : hourRanges) {
                int cant = messageJDBCRepository.countMessageInRangeHour(hourRange.getStartDate(), hourRange.getEndDate());
                int rangeHour = getRangeHour(hourRange.getStartDate().getHour());
                Long timestamp = getTimestamp(hourRange.getStartDate());
                if (!accumulated.containsKey(timestamp)) {
                    accumulated.put(timestamp, new HashMap<>());
                }
                if (!accumulated.get(timestamp).containsKey(rangeHour)) {
                    accumulated.get(timestamp).put(rangeHour, 0);
                }
                accumulated.get(timestamp).put(rangeHour, accumulated.get(timestamp).get(rangeHour) + cant);
            }
        }

        accumulated.keySet().stream().forEach(x -> {
            accumulated.get(x).keySet().stream().forEach(y -> {
                data.add(new Object[]{x, y, accumulated.get(x).get(y)});
            });
        });
        return data;
    }

    private int getRangeHour(int hour) {
        switch (hour) {
            case 3:
            case 4:
            case 5:
                return 1;

            case 6:
            case 7:
            case 8:
                return 2;

            case 9:
            case 10:
            case 11:
                return 3;

            case 12:
            case 13:
            case 14:
                return 4;

            case 15:
            case 16:
            case 17:
                return 5;

            case 18:
            case 19:
            case 20:
                return 6;

            case 21:
            case 22:
            case 23:
                return 7;

            default:
                return 0;
        }
    }

    private long getTimestamp(OffsetDateTime date) {
        OffsetDateTime dateTime;
        dateTime = date.withHour(0);
        dateTime = dateTime.withMinute(0);
        dateTime = dateTime.withSecond(0);
        dateTime = dateTime.withNano(0);
        return dateTime.toInstant().toEpochMilli();
    }

    private List<RangeBean> getRangeHours(OffsetDateTime startDate, OffsetDateTime endDate) {
        OffsetDateTime nextDate = getNextHour(startDate);
        OffsetDateTime startNextDate = startDate;
        List<RangeBean> rangeList = new ArrayList<>();
        while (nextDate.isBefore(endDate)) {
            RangeBean rangeBean = new RangeBean();
            rangeBean.setStartDate(startNextDate);
            rangeBean.setEndDate(nextDate);
            rangeList.add(rangeBean);
            startNextDate = nextDate;
            nextDate = getNextHour(nextDate);
        }
        if (startNextDate.isBefore(endDate)) {
            RangeBean rangeBean = new RangeBean();
            rangeBean.setStartDate(startNextDate);
            rangeBean.setEndDate(endDate);
            rangeList.add(rangeBean);
        }
        return rangeList;
    }
}
